C**********************************************************************
C
C     Program drops
C
C     Purpose:  simulates powder bed via random raindrop algorithm.
C               Raindrops are hard, incompressible discs.  Simulation 
C               bed is of dimensions xmax X zxmax.  Simulation
C               is initially 2D, 3D to be added later.
C
C     Key Variable list:
C
C       x = x position of particle
C       z = height of particle from arbitrary datum
C       R = radius of particle
C       type  = 1: Simulation of bimodal particle size distr.
C               2: Gaussian particle size distribution
C               3: Particle size sample from Beta distribution.
C       xmax  = max x value of simulation box
C       zmax  = max particle height
C       counter = total number of particles
C       i, j, k = loop counters
C       mean_u, mean_l = ave particle size, upper and lower
C       sigma = (type 1) ratio of large to small particles
C               (type 2) std dev of normal distribution
C
C    G(r) plot variables:
C
C       dR    = radius increments for G(r) plot
C       dRmax = maximum radius to be considered
C       partmax = max number of particles allowed in simulation
C       bin     = radius increment for G(r) histogram
C       histogram      = no. of counts in each bin
C       ideal_constant = PI/2*area (to calculate Nid)
C       Nbin, Nid = average no. counts for model and ideal case
C       Gr = particle pair distribution function at dR intervals
C
C     Input files : 
C
C       drops.dat        initialisation data
C
C     Output files :
C
C       count.out       G(r) plot
C       position1.out   initial particles positions and radii
C       circles.dat     gnuplot data (optional)
C       check.out       overlapping particles info
C       temp.out        'neighbourhood' particles (disabled)
C
C     NAG Routines :
C
C       G05CAF  Random number from 0 to 1 (uniform dist.)
C       G05CCF  Initialises random number generation
C       G05DAF  Random number from specified min to max
C               (uniform distribution)
C       G05DDF  Random number from normal dist. (specify mean
C               and std dev.)
C       G05DYF  Integer random number from specified min to max
C               (uniform dist.)
C       G05FEF  Random number from Beta distribution (specify 
C               parameters a and b.
C 
C**********************************************************************
       program rain_drops
       implicit none
C
C----------------------------------------------------------------------
C     Initialise variables
C
       integer counter, i
       integer partmax, type, new
       integer check, last, max, degrees
       parameter (max = 5000, degrees = 359)

       double precision x(max), z(max), R(max), o(max)
       double precision xmax, zmax, xrand
       double precision mean_u, mean_l, dprt
       double precision sigma, dR, dRmax, inset
       double precision PI, radians, epsilon
       parameter (PI = 3.141592654, radians = PI/180.0D0)
C
       data counter, last, check/3*0/
C
       common /particles/ x(max), z(max), R(max), o(max)
       common /bounds/ mean_u, mean_l, xmax, zmax
       common /rad/ dR, dRmax
       common /shapes/  epsilon, inset
C
       external G05CAF, G05CCF
       external G05DAF, G05DDF
       external G05DYF, G05FEF
C
       double precision G05CAF, G05DAF, G05DDF
       integer G05DYF
       call G05CCF(0)
C
       open(10,file='drops.dat')
       open(12,file='position1.out')
       open(16,file='temp.out')
C
       read(10,*) xmax, zmax, mean_u, mean_l
       read(10,*) sigma, dR, dRmax, inset
       read(10,*) partmax, type, epsilon
C
C----------------------------------------------------------------------
C  Random rain drops
C
       write(6,*) 'Generating random particle deposition .....'
C
       do 5, counter = 1, partmax
        new = counter
C
C  Get random radius
C
        if (type.eq.1)  then
           call bimodal(sigma,dprt)
        else if (type.eq.2)  then
           call normal(sigma,dprt)
        else if (type.eq.3)  then
           call beta(dprt)
        end  if
C
C  Get random x position, assign coordinates
C
7       continue
        xrand  = G05DAF(0.0D0,xmax)
        x(new) = xrand
        R(new) = dprt
        o(new) = dble(G05DYF(0, degrees))*radians
C
C  Find zposition for given x
C
        if (new.ne.1)  then
           call zposition(new)
        else
           z(new) = R(new)
        end  if
C
C   Mark progress in units of 50
C
        last = mod(new, 50)
        if (last.eq.0)  write(6,*) 'Done ', new
C
C  Check for particle height > zmax
C
C        if (z(new).gt.zmax)  goto 1000
C
5      continue
C
C---------------------------------------------------------------------
C   Output no. of particles to file position1.out
C
1000   check = counter
       if (check.gt.partmax) check = partmax
       write(12,*) check
       write(6,*) 'Counter is ', check
C
C-----------------------------------------------------------------------
C   Output results and data to position1.out
C
       do 2000, i = 1, check
        write(12,*) i,'  ', x(i),'  ', z(i),'  ', R(i), '  ', o(i)
2000   continue
C
       write(12,*)
       write(12,*) xmax, zmax, mean_u, mean_l
       write(12,*) sigma, dR, dRmax, inset
       write(12,*) partmax, type, epsilon
C-----------------------------------------------------------------------
C   Close files and end program
C
       write(6,*) 'Done'
       close (10)
       close (12)
       close (13)
       close (16)
       stop
       end
C*****************************************************************
C
C  Subroutine bimodal
C
C    Purpose :  generates random particles from bimodal
C               distribution using NAG routines
C 
C    Input Arguments : 
C
C         mean_u  particle mean radius (upper)
C         mean_l     "       "     "   (lower)
C         sigma   particle size ratio (large:small)
C
C    Output Arguments
C
C         dprt    particle size randomly chosen
C
C    Subroutines :
C
C    Called from :
C
C         main
C
C    Calls to :
C
C         none
C
C*****************************************************************
       subroutine bimodal(sigma,dprt)
       double precision mean_u, mean_l, sigma, dprt
       double precision constant, xmax, zmax
       double precision G05CAF
C
       common /bounds/ mean_u, mean_l, xmax, zmax
C
       constant = G05CAF(constant)
       if (mean_u.eq.mean_l)  then
          dprt = mean_u
       else
          if (constant.le.sigma)  then
             dprt = mean_u
          else
             dprt = mean_l
          end  if
       end  if
C
       return
       end
C*****************************************************************
C
C  Subroutine beta
C
C    Purpose :  generates random particles from beta
C               distribution
C 
C    Input Arguments : 
C
C         none    common block and data file used
C
C    Output Arguments
C
C         dprt    particle size randomly chosen
C
C    Common Blocks :
C
C         /bounds/   mean_u, mean_l, xmax, zmax
C
C    Subroutines :
C
C    Called from :
C
C         main
C
C    Calls to :
C
C         G05CAF  NAG routine for generating variates from a 
C                 uniform distribution.
C
C*****************************************************************
       subroutine beta(dprt)
       integer i, incr, G05DYF
       integer start, stop
       integer top, constant
       parameter (top = 1001)
C
       double precision mean_u, mean_l
       double precision dprt, xmax, zmax
       double precision a, b, num
       double precision max, min, dif
       double precision total, index, first, second
       double precision val(top), cum(top)
       double precision steps, value
       double precision left, right, temp
       double precision box, triangle, area, height
       double precision product, result, cumul
       parameter (steps = 1.0D-3)
C
       common /bounds/ mean_u, mean_l, xmax, zmax
       save  cum
C
C  Open file 'beta.inp' and read beta function constants
C
       open(14,file='beta.inp')
       read(14,*) a, b
C
C  Initialise variables
C
       total    = 0.0D0
       start    = 0
       incr     = 0
       cumul    = 0.0D0
       stop     = int(1.0D0/steps)
       constant = G05CAF(constant)
C
C  Evaluate Beta integral numerically
C
       do 10, i = start, stop
        incr    = incr+1
        index   = a-1.0D0
        if (index.eq.0.0D0)  then
           first = 1.0D0
        else
           first   = (i*steps)**index
        end if
        index   = b-1.0D0
        if (index.eq.0.0D0)  then
           second = 1.0D0
        else
           value   = 1.0D0-(i*steps)
           second  = value**index
        end  if
        val(incr) = first*second
C
        if (incr.gt.1)  then
           left = val(incr-1)
           right = val(incr)
           if (left.gt.right)  then
              temp  = left
              left  = right
              right = temp
           end  if
           box = left * steps
           height = right-left
           triangle = (steps*height)/2.0D0
           area = box + triangle
           total   = total + area
        end  if
10     continue
C
C  Calculate beta pdf ('result') and cdf('cum')
C
       do 20, i = start, stop
        index = a-1.0D0
        if (index.eq.0.0D0)  then
            first = 1.0D0
        else
            first = (i*steps)**index
        end if
        index = b-1.0D0
        if (index.eq.0.0D0)  then
           second = 1.0D0
        else
           value  = 1.0D0-(i*steps)
           second = value**index
        end  if
        product  = first*second
        result   = product/total
        cumul    = cumul + result
        cum(i+1) = cumul/stop
20     continue
C
C  Convert to particle radius
C
       constant = G05DYF(1, top)
       num      = cum(constant)
C       write(6,*) num
C
C  Upper limit set to mean_u*1.5; lower limit mean_l/10
C
       max = mean_u*1.5D0
       min = mean_l/10.0D0
       dif = max - min
       dprt  = (dif*num) + min
C       write(6,*) dprt
C
       close(14)
       return
       end
C*****************************************************************
C
C  Subroutine normal
C
C    Purpose :  generates random particles from normal
C               (Gaussian) distribution using NAG routines
C 
C    Input Arguments : 
C
C         sigma   particle size distribution standard deviation
C
C    Output Arguments :
C
C         dprt    particle size randomly chosen
C
C    Common Blocks :
C
C         /bounds/  mean_u, mean_l, xmax, zmax
C
C    Subroutines :
C
C    Called from :
C
C         main
C
C    Calls to :
C
C         none
C
C*****************************************************************
       subroutine normal(sigma,dprt)
       double precision G05DDF, constant
       double precision mean_u, mean_l, xmax, zmax
       double precision sigma, dprt, err
C
       common /bounds/ mean_u, mean_l, xmax, zmax
C
1001   continue
       constant = G05DDF(mean_u,sigma)
       dprt = constant
       if (dprt.lt.0.01D0) dprt = 0.01D0
C
C  Rechoose if radius > mean + err ('Error factor').
C     err set = sigma
C
       err = sigma
       if (dprt.gt.(err+mean_u))  goto 1001
C
       return
       end
C*****************************************************************
C
C  Subroutine link_len
C
C    Purpose :  Calculate distance between any two particle centres
C 
C    Input Arguments : 
C
C         x1, z1   Coordinates of particle centres
C         x2, z2
C
C    Output Arguments :
C
C         hypo     Distance between particle centres
C
C    Subroutines :
C
C    Called from :
C
C         many
C
C    Calls to :
C
C         none
C
C*****************************************************************
       subroutine link_len(x1, z1, x2, z2, hypo)
       double precision x1, z1, x2, z2, hypo
       double precision xdistance, zdistance
C
       xdistance = (x2-x1)**2
       zdistance = (z2-z1)**2
       hypo = sqrt(xdistance+zdistance)
C
       return
       end
C*****************************************************************
C
C  Subroutine zposition
C
C    Purpose :  generates particle z position               
C 
C    Input Arguments : 
C
C         new     No. of current particle to be positioned
C
C    Output Arguments :
C
C         none    writes to variables in common blocks
C
C    Common Blocks :
C
C         /particles/  x, z, R, o
C         /bounds/     mean_u, mean_l, xmax, zmax
C
C    Subroutines :
C
C    Called from :
C
C         main
C
C    Calls to :
C
C         roll
C         hpsort
C
C*****************************************************************
       subroutine zposition(new)
       integer new, hit, miss
       integer incr, max, kilo
       parameter (max = 5000, kilo = 1000)
       double precision x(max), z(max), R(max), o(max)
       double precision xmax, zmax
       double precision xupper, xlower, zlower
       double precision xpos1, zpos1, xpos2, zpos2
       double precision xcent(kilo), zcent(kilo)
       double precision ocent(kilo), Rcent(kilo)
       double precision hypo, dist, mean_l
       double precision decrement, radius1, mean_u
       double precision new_dims(4), modifier
C
       data hit, miss/2*0/
       data decrement, modifier/0.1D0, 2.0D0/
C
       common /particles/ x(max), z(max), R(max), o(max)
       common /bounds/ mean_u, mean_l, xmax, zmax
C
C  Set start z position = zmax + 10
C
       dist = zmax+10.0D0
C
C  Set bounds for neighbourhood particles
C
       xlower = x(new) - (mean_u*5.0D0)
       xupper = x(new) + (mean_u*5.0D0)
       zlower = z(new) - (mean_u*5.0D0)
       if (xlower.lt.0.0D0)  then
          xlower = 0.0D0
       end  if
       if (xupper.gt.xmax)  then
          xupper = xmax
       end  if
       if (zlower.lt.0.0D0)  zlower = 0.0D0
C
C  Get coords of particles within x bounds
C
       if (new.gt.1)  then
          do 9, i = 1, (new-1)
           if (x(i).ge.xlower.and.x(i).le.xupper)  then
              if (z(i).ge.zlower)  then
                 hit = hit+1
                 xcent(hit) = x(i)
                 zcent(hit) = z(i)
                 Rcent(hit) = R(i)
                 ocent(hit) = o(i)
              end  if
           end  if
9         continue
       end  if
C----------------------------------------------------------------------
C  For hit > 1 :
C  Sort particles in ascending z order
C
       if (hit.gt.1)  then
          call hpsort(hit, zcent, xcent, Rcent, ocent)
C
C  Write contents of xcent, zcent, Rcent and ocent to 'temp.out'
C     [DISABLED]
C
C       write(6,*) hit
C          write(16,*) x(new)
C          write(16,*) hit
C          do 12, i = 1, hit
C           write(16,*) i, xcent(i), zcent(i), Rcent(i), ocent(i)
C12        continue
C          write(16,*)
       end  if
C----------------------------------------------------------------------
C
       if (hit.lt.1)  then  !!  No particles in bounds !!
C          write(6,*) 'Hit is :', hit, ' New is :', new
          z(new) = R(new)
       else if (hit.ge.1)  then  !!  One or more particles in bounds !!
C          write(6,*) 'Hit is :', hit, ' New is :', new
25        z(new) = dist
C          write(6,*) 'z(new) is :', z(new)
C
C  Check all n'hood particles vs 'new'.  If overlap found, set miss = 1
C
          do 22, j = 1, hit
           xpos1 = xcent(j)
           zpos1 = zcent(j)
           xpos2 = x(new)
           zpos2 = z(new)
           radius1 = R(new)+Rcent(j)
           call link_len(xpos1, zpos1, xpos2, zpos2, hypo)
           if (hypo.le.radius1)  then
              miss = 1
           end  if
22        continue
C
C  If miss = 0, decrement 'dist' and try again.  Otherwise, reduce 
C    'decrement' and try again.  On fifth failure, accept z position.
C
          if (miss.eq.0)  then
             dist = dist - decrement
             if (dist.gt.R(new))  then
C             write(6,*) dist
                goto 25
             else
                z(new) = R(new)
             end  if
          else
             dist = dist + decrement
             incr = incr + 1
             miss = 0
             if (incr.lt.5)  then
                decrement = decrement/modifier
                dist = dist - decrement
                goto 25
             else
                z(new) = dist
             end  if
          end  if
       end  if
C
C  Call rolling raindrops routine if hit > 0 and z(new) > mean_u
C
       if (hit.gt.0.and.z(new).gt.mean_u)  then
          new_dims(1) = x(new)
          new_dims(2) = z(new)
          new_dims(3) = R(new)
          new_dims(4) = o(new)
          call roll(new_dims, xcent, zcent, Rcent, ocent, hit, new)
C
C  Reset x and z new values with roll displacement values
C
          x(new) = new_dims(1)
          z(new) = new_dims(2)
       end  if
C
C   Clear 'cent' array
C
       do 15, i = 1, hit
        xcent(i) = 0.0D0
        zcent(i) = 0.0D0
        Rcent(i) = 0.0D0
        ocent(i) = 0.0D0
15     continue
       hit  = 0
       miss = 0
C       write(6,*)
C
       return
       end
C*****************************************************************
C
C  Subroutine hpsort
C
C    Purpose :  sorts particle list using HEAPSORT algorithm
C               (as per Press et al, Ch 8) 
C 
C    Input Arguments : 
C
C         ra, rb, rc, rd  x, y, R and o values to be sorted
C                         ** ra is sort key **
C         n               no. of particles in sort list
C
C    Output Arguments
C
C         ra, rb, rc  sorted particle list
C
C    Subroutines :
C
C    Called from :
C
C         zposition
C
C    Calls to :
C
C         none
C
C*****************************************************************
       subroutine hpsort(n, ra, rb, rc, rd)
       integer n
       double precision ra(n), rb(n), rc(n), rd(n)
       integer i, ir, j, l
       double precision rra, rrb, rrc, rrd
C
       if(n.lt.2)  return
       l = n/2+1
       ir = n
C
10     continue
        if (l.gt.1)  then
           l = l-1
           rra = ra(l)
           rrb = rb(l)
           rrc = rc(l)
           rrd = rd(l)
        else
           rra = ra(ir)
           rrb = rb(ir)
           rrc = rc(ir)
           rrd = rd(ir)
           ra(ir) = ra(1)
           rb(ir) = rb(1)
           rc(ir) = rc(1)
           rd(ir) = rd(1)
           ir = ir-1
           if (ir.eq.1)  then
              ra(1) = rra
              rb(1) = rrb
              rc(1) = rrc
              rd(1) = rrd
              return
           end  if
        end  if
        i = l
        j = l+l
20      if (j.le.ir)  then
           if (j.lt.ir)  then
              if (ra(j).lt.ra(j+1))  j = j+1
           end if
           if (rra.lt.ra(j))  then
              ra(i) = ra(j)
              rb(i) = rb(j)
              rc(i) = rc(j)
              rd(i) = rd(j)
              i = j
              j = j+j
           else
              j = ir+1
           end  if
        goto 20
        end  if
        ra(i) = rra
        rb(i) = rrb
        rc(i) = rrc
        rd(i) = rrd
       goto 10
       end
C*****************************************************************
C
C  Subroutine roll
C
C    Purpose :  generates particle z position using 'rolling 
C               raindrop' algorithm
C 
C    Input Arguments : 
C
C         new_dims  x, z, R and o values of current new particle
C         xcent -
C         zcent  >  x, z, R and o values of particles within bounds
C         Rcent -
C         hit       number of particles within bounds 
C
C    Output Arguments
C
C         none      writes to common block
C
C    Common Blocks
C
C         /particles/  x, z, R, o
C
C
C    Subroutines :
C
C    called from :
C
C         zposition
C
C    calls to :
C
C         link_len
C         friction
C
C*****************************************************************
       subroutine roll(newdims, xcent, zcent, Rcent, ocent, hit, new)
       integer hit, o_lap, incr, incr2
       integer new, neg, max, kilo
       integer one, zero
       parameter (max = 5000, kilo = 1000)
       parameter (one = 1, zero = 0)
C
       double precision newdims(4)
       double precision x(max), z(max), R(max), o(max)
       double precision xcent(kilo), zcent(kilo)
       double precision ocent(kilo), Rcent(kilo)
       double precision xpos1, zpos1, xpos2, zpos2
       double precision Rpos1, Rpos2, hypo
       double precision current
       double precision deltax, deltaz, minimum, angle_deg
       double precision angle_rad, tan_angle, radians
       double precision markerx, markerz, markerR
       double precision newmarkx, newmarkz, newmarkR
       double precision polarx, polarz, top, PI
       double precision xnew, znew, Rnew, onew, xold, zold
       double precision radius, rotation, new_rot
       parameter (PI = 3.141592654)
       parameter (radians = PI/180.0D0)
C
       data o_lap, neg, incr, incr2/4*0/
C
       common /particles/ x(max), z(max), R(max), o(max)
C
       rotation = PI/(180.0D0*4.0D0)
       xnew     = newdims(1)
       znew     = newdims(2)
       Rnew     = newdims(3)
       onew     = newdims(4) 
C
C  Find closest existing particle to 'new' within 'hit' bounds
C
       top = 100.0D0
       do 300, i = 1, hit
        xpos2 = xcent(i)
        zpos2 = zcent(i)
        call link_len(xnew, znew, xpos2, zpos2, hypo)
        if (hypo.lt.top)  then
           top = hypo
           markerx = xcent(i)
           markerz = zcent(i)
           markerR = Rcent(i)
        end  if
300    continue
C
       minimum = markerR+Rnew
C
22     call link_len(xnew, znew, markerx, markerz, hypo)
       radius = hypo
       deltax = xnew - markerx
       deltaz = znew - markerz
C       write(6,*) 'Delta x : ', deltax, ' Delta z : ', deltaz
C       write(6,*) markerx, markerz, ' Markers !'
C       write(6,*) xnew, znew
C
C  If deltax is -ve, set neg = 1 (ie angle > 90 degrees)
C
       if (deltax.lt.0.0D0)  then
          neg = one
       else
          neg = zero
       end  if
C
C  Calculate angle in radians
C
       tan_angle = deltaz/deltax
       angle_rad = atan(tan_angle)
       if (neg.eq.1)  angle_rad = PI + angle_rad
       angle_deg = angle_rad*(1/radians)
C       if (neg.eq.1)  angle_deg = (2*PI)+angle_deg
C       write(6,*) 'Angle is ', angle_deg, ' deg'
C
C  Start angle reduction loop:  reduce angle by one deg. and
C    recalculate position
C
       new_rot = rotation
24     continue
       if (neg.eq.one)  then  !! Angle > 90 deg !!
          angle_rad = angle_rad+new_rot
       else                 !! Angle < 90 deg !!
          angle_rad = angle_rad-new_rot
       end  if
       angle_deg = angle_rad*(1/radians)
C       write(6,*) 'Angle is ', angle_deg, ' deg'
       polarx = radius*(cos(angle_rad))
       polarz = radius*(sin(angle_rad))
C
C  Recalculate position of 'new' from polars.
C
       xpos1 = markerx+polarx
       zpos1 = markerz+polarz
       Rpos1 = Rnew
C       write(6,*) xpos1, zpos1
C
C  Check for particle overlap; if found, set o_lap = one
C
C       do 30, i = 1, hit
       do 30, i = 1, (new-1)
C        xpos2 = xcent(i)
C        zpos2 = zcent(i)
C        Rpos2 = Rcent(i)
        xpos2 = x(i)
        zpos2 = z(i)
        Rpos2 = R(i)
        call link_len(xpos1, zpos1, xpos2, zpos2, hypo)
        if (hypo.lt.(Rpos1+Rpos2))  then
           o_lap = one
           newmarkx = x(i)
           newmarkz = z(i)
           newmarkR = R(i)
        end  if
30     continue
C
C  If no o_lap = 0, rotate 1 deg. and try again; if o_lap = 1, 
C    reduce rotation and try again, until tries = 5.
C
       xold = xpos1   !! xpos1 = markerx+polarx  !!
       zold = zpos1   !!      ditto zpos1        !!
       if (o_lap.eq.zero)  then
          if (zold.gt.(markerz-(Rnew+markerR)))  then
             call friction(current, angle_rad, stick) 
             if (stick.eq.0)  then
                goto 24
             else
                current = 0.0D0
                xnew    = xold
                znew    = zold
                goto 9999
             end  if
          else
             current = 0.0D0
             xnew    = xold
             znew    = zold
             goto 9999
          end  if
       else
          o_lap = zero
          if (neg.eq.one)  then
             angle_rad = angle_rad-new_rot
          else
             angle_rad = angle_rad+new_rot
          end  if
          incr = incr + one
          if (incr.lt.5)  then
C             write(6,*) 'Fail'
             new_rot = new_rot/2.0D0
             goto 24
          else
             polarx = radius*(cos(angle_rad))
             polarz = radius*(sin(angle_rad))
             xnew   = markerx+polarx
             znew   = markerz+polarz
             if (newmarkz.lt.znew.and.newmarkz.ne.mean_u)  then
                incr2 = incr2 + one
                if (incr2.lt.3)  then
                   o_lap = zero
                   markerx = newmarkx
                   markerz = newmarkz
                   markerR = newmarkR
                   goto 22
                end  if
             end  if
C             write(6,*)
          end  if
       end  if
C
9999   continue
C       write(6,*)
C
C  Write new coordinates into 'newdims' array
C
       newdims(1) = xnew
       newdims(2) = znew
       neg   = zero
       incr  = zero
       incr2 = zero
C
       return
       end
C*****************************************************************
C
C  Subroutine friction
C
C    Purpose :  Simulated effect of friction on particle 
C               interaction
C 
C    Input Arguments : 
C
C         angle     relative particle angle
C
C    Output Arguments
C
C         stick     set to 1 if 'stuck', 0 otherwise
C
C
C    Subroutines :
C
C    called from :
C
C         roll
C
C    calls to :
C
C         link_len
C
C*****************************************************************
       subroutine friction(current, angle, stick)
       double precision angle, angle_deg
       double precision max, current
       double precision PI, radians
       double precision G05CAF, number
       double precision increment
       parameter (PI = 3.1415926, radians = PI/180.0D0)
       parameter (max = 15.0D0, increment = max/180.0D2)
C
       integer stick
C
       stick = 0
       angle_deg = angle/radians
C       write(6,*) angle, angle_deg
       number = G05CAF(number)
       current = current+increment
       if (number.lt.current)  then
          stick = 1
C          write(6,*) current
       end  if
C
       return
       end
C*****************************************************************
